@@ -8,14 +8,15 @@
 !-----------------------------------------------------------------------------
 MODULE cp_restart_new
   !-----------------------------------------------------------------------------
+  !! This module contains subroutines to write and read data required to
+  !! restart a calculation from the disk.
+  !! Important notice:
   !
-  ! ... This module contains subroutines to write and read data required to
-  ! ... restart a calculation from the disk. Important notice:
-  ! ... * only one processor writes (the one for which ionode = .true.)
-  ! ... * all processors read the xml file
-  ! ... * one processor per band group reads the wavefunctions,
-  ! ...   distributes them within their band group
-  ! ... * lambda matrices are read by one processors, broadcast to all others
+  !! * only one processor writes (the one for which ionode=.TRUE.);
+  !! * all processors read the xml file;
+  !! * one processor per band group reads the wavefunctions distributes 
+  !!   them within their band group;
+  !! * lambda matrices are read by one processor, broadcast to all others.
   !
   USE kinds,     ONLY : DP
   !
@@ -28,14 +29,14 @@ MODULE cp_restart_new
   USE io_global, ONLY : ionode, ionode_id, stdout
   USE mp,        ONLY : mp_bcast
   USE matrix_inversion
-#if defined (__outfoxed)
-    USE     wxml
-    USE     dom,     ONLY : Node, parseFile, item, getElementsByTagname, &
+#if defined (__fox)
+    USE FoX_wxml
+    USE FoX_dom,     ONLY : Node, parseFile, item, getElementsByTagname, &
                             hasAttribute, extractDataAttribute, &
                             extractDataContent, destroy
 #else
-    USE FoX_wxml
-    USE FoX_dom,     ONLY : Node, parseFile, item, getElementsByTagname, &
+    USE     wxml
+    USE     dom,     ONLY : Node, parseFile, item, getElementsByTagname, &
                             hasAttribute, extractDataAttribute, &
                             extractDataContent, destroy
 #endif
@@ -564,6 +565,7 @@ MODULE cp_restart_new
                             lambda0, lambdam, b1, b2, b3, xnhe0, xnhem, vnhe, &
                             ekincm, c02, cm2, wfc )
       !------------------------------------------------------------------------
+      !! Read XML file.
       !
       USE control_flags,            ONLY : gamma_only, force_pairing, llondon,&
                                            ts_vdw, mbd_vdw, lxdm, iverbosity, lwf
@@ -685,6 +687,8 @@ MODULE cp_restart_new
       LOGICAL :: backall_dum(nsp)
       INTEGER :: hub_l2_dum(nsp), hub_l3_dum(nsp), hub_lmax_back_dum  
       CHARACTER(LEN=6), EXTERNAL :: int_to_char
+      INTEGER  :: int_dum 
+      LOGICAL  :: bool_dum
       !
       !
       dirname = restart_dir(ndr)
@@ -782,10 +786,11 @@ MODULE cp_restart_new
       CALL qexsd_copy_dft ( output_obj%dft, nsp, atm, dft_name, &
            nq1, nq2, nq3, ecutfock, exx_fraction, screening_parameter, &
            exxdiv_treatment, x_gamma_extrapolation, ecutvcut, local_thr, &
-           lda_plus_U, lda_plus_U_kind, Hubbard_projectors, Hubbard_n, Hubbard_l, Hubbard_lmax,&
+           lda_plus_U, lda_plus_U_kind, Hubbard_projectors, Hubbard_n, Hubbard_l, Hubbard_lmax,Hubbard_dum, &
            hub_l2_dum, hub_l2_dum, hub_l2_dum, hub_l2_dum, backall_dum, hub_lmax_back_dum, hubba_dum, & 
            Hubbard_U, hubba_dum, Hubbard_dum(1,:), Hubbard_dum(2,:), Hubbard_dum(3,:), &
-           HUBBARD_J = Hubbard_dum, HUBBARD_V = hubba_dum_dum, VDW_CORR = vdw_corr, SCAL6 = scal6,    & 
+           HUBBARD_J = Hubbard_dum, HUBBARD_V = hubba_dum_dum, VDW_CORR = vdw_corr, dftd3_version = int_dum, & 
+           dftd3_3body = bool_dum, SCAL6 = scal6,    & 
            LON_RCUT =lon_rcut, VDW_ISOLATED = vdw_isolated )
       CALL set_vdw_corr (vdw_corr, llondon, ldftd3, ts_vdw, mbd_vdw, lxdm )
       IF ( ldftd3 ) CALL errore('cp_readfile','DFT-D3 not implemented',1)
@@ -843,7 +848,7 @@ MODULE cp_restart_new
        xnhp0, vnhp, ekincm, xnhe0, vnhe, ht, htvel, gvel, xnhh0, vnhh,      &
        staum, svelm, xnhpm, xnhem, htm, xnhhm) !
     !------------------------------------------------------------------------
-    ! ... Cell related variables, CP-specific
+    !! Cell related variables, CP-specific.
     !
     USE ions_base, ONLY: nat
     !
@@ -1067,8 +1072,7 @@ MODULE cp_restart_new
   !------------------------------------------------------------------------
   SUBROUTINE cp_writecenters( xf, h, wfc )
     !------------------------------------------------------------------------
-    !
-    ! ... Write Wannier centers
+    !! Write Wannier centers.
     !
     USE kinds, ONLY : dp
     USE io_global, ONLY : ionode
@@ -1117,9 +1121,10 @@ MODULE cp_restart_new
   !------------------------------------------------------------------------
   SUBROUTINE cp_read_wfc( ndr, ik, nk, iss, nspin, c2, tag, ierr )
     !------------------------------------------------------------------------
-    !
-    ! Wrapper, and ugly hack, for old cp_read_wfc called in restart.f90
-    ! If ierr is present, returns ierr=-1 if file not found, 0 otherwise
+    !! Wrapper, and ugly hack, for old \(\texttt{cp_read_wfc}\) called in 
+    !! \(\texttt{restart.f90}\).  
+    !! If \(\text{ierr}\) is present, returns (\text{ierr}=-1\) if file not
+    !! found, 0 otherwise.
     !
     USE mp_bands,           ONLY : me_bgrp, root_bgrp, intra_bgrp_comm
     USE electrons_base,     ONLY : iupdwn, nupdwn
@@ -1171,12 +1176,13 @@ MODULE cp_restart_new
        ierr )
     !
     !------------------------------------------------------------------------
-    ! ... Cell related variables, CP-specific
-    ! ... ierr = -2: nothing found
-    ! ... ierr = -1: MD status found, no info on timesteps
-    ! ... ierr =  0: MD status and timestep info read
-    ! ... ierr =  1: error reading MD status
-    ! ... ierr =  2: error reading timestep info
+    !! Cell related variables, CP-specific:
+    !
+    !! * ierr = -2: nothing found;
+    !! * ierr = -1: MD status found, no info on timesteps;
+    !! * ierr =  0: MD status and timestep info read;
+    !! * ierr =  1: error reading MD status;
+    !! * ierr =  2: error reading timestep info.
     !
     !
     TYPE(Node),POINTER,INTENT(IN) :: root
@@ -1385,8 +1391,7 @@ MODULE cp_restart_new
   !------------------------------------------------------------------------
   SUBROUTINE cp_readcenters( root, wfc )
     !------------------------------------------------------------------------
-    !
-    ! ... Read Wannier centers
+    !! Read Wannier centers.
     !
     USE kinds, ONLY : dp
     USE io_global, ONLY : stdout
@@ -1565,8 +1570,7 @@ MODULE cp_restart_new
   SUBROUTINE cp_write_lambda( filename, iunpun, iss, nspin, nudx, &
        lambda, ierr )
     !------------------------------------------------------------------------
-    !
-    ! ... collect and write matrix lambda to file
+    !! Collect and write matrix lambda to file.
     !
     USE kinds, ONLY : dp
     USE mp, ONLY : mp_bcast
@@ -1604,8 +1608,7 @@ MODULE cp_restart_new
   SUBROUTINE cp_read_lambda( filename, iunpun, iss, nspin, nudx, &
              lambda, ierr )
     !------------------------------------------------------------------------
-    !
-    ! ... read matrix lambda from file, distribute it
+    !! Read matrix lambda from file, distribute it.
     !
     USE kinds, ONLY : dp
     USE mp, ONLY : mp_bcast
@@ -1647,8 +1650,7 @@ MODULE cp_restart_new
   !------------------------------------------------------------------------
   SUBROUTINE cp_write_zmat( ndw, mat_z, ierr )
     !------------------------------------------------------------------------
-    !
-    ! ... collect and write matrix z to file
+    !! Collect and write matrix z to file.
     !
     USE kinds, ONLY : dp
     USE mp, ONLY : mp_bcast
@@ -1696,8 +1698,7 @@ MODULE cp_restart_new
   !------------------------------------------------------------------------
   SUBROUTINE cp_read_zmat( ndr, mat_z, ierr )
     !------------------------------------------------------------------------
-    !
-    ! ... read from file and distribute matrix z
+    !! Read from file and distribute matrix z.
     !
     USE kinds, ONLY : dp
     USE mp, ONLY : mp_bcast
