@@ -118,6 +118,8 @@
   !! if .TRUE. print the |g| vertex in [meV].
   LOGICAL :: lphase
   !! if .TRUE. fix the gauge when diagonalizing the interpolated dynamical matrix and electronic Hamiltonian.
+  LOGICAL :: lrot
+  !! if .TRUE. fix the gauge when diagonalizing the interpolated dynamical matrix and electronic Hamiltonian.
   LOGICAL :: lindabs
   !! if .TRUE. perform phonon-assisted absorption calculations
   LOGICAL :: use_ws
@@ -367,6 +369,31 @@
   !! Magnetic field along the z-direction
   REAL(KIND = DP) :: mob_maxfreq
   !! Maximum frequency for the spectral decomposition of mobility. Typically that freq. is the highest phonon freq.
+  !!!!!
+  !
+  ! Conductivity - ionized impurity scattering
+  LOGICAL :: ii_g
+  !! If .TRUE. calculate the ionized impurity matrix elements
+  LOGICAL :: ii_scattering
+  !! If .TRUE. calculate the carrier-impurity scattering rate
+  LOGICAL :: ii_only
+  !! If .TRUE., calculate only the ionized-impurity-limitted mobility
+  LOGICAL :: ii_lscreen
+  !! If .TRUE. (DEFAULT), calculate and use free-carrier
+  !! screening of ii_g matrix elements, if .FALSE., calculations will diverge with 
+  !! increasing density of k-point due to 1/(q^4) divergence for ionized impurity 
+  !! scattering rate
+  LOGICAL :: ii_partion
+  !! If .TRUE., account for partial ionization 
+  REAL(KIND = DP) :: ii_charge
+  !! charge of the ionized impurities, units of electron charge for input
+  REAL(KIND = DP) :: ii_n
+  !! density of impurities, input in units of cm^-3
+  REAL(KIND = DP) :: ii_eda
+  !! ionization energy of the donor or accpetor impurity, for partial ionization calcs
+  REAL(KIND = DP) :: ii_eps0
+  !! low frequency dielectric constant
+  !!!!! 
   !
   ! Plasmon
   REAL(KIND = DP) :: nel
@@ -391,74 +418,111 @@
   !! Photon energy maximum (in eV)
   REAL(KIND = DP) :: omegastep
   !! Photon energy step (in eV)
-  REAL(KIND = DP) :: n_r
-  !! Refractive index
+  REAL(KIND = DP) :: sigma_ref
+  !! Reference conductivity for Drude dielectric function
   !
   ! ----------------------------------------------------------------------------------
   ! Added for polaron calculations. Originally by Danny Sio, modified by Chao Lian.
   ! Shell implementation for future use.
-  INTEGER :: num_cbands
-  !! number of conduction bands accounted in the Hilbert space of polaron Hamilontian
-  INTEGER :: start_band
-  !! start band index in matrix element
-  INTEGER :: nPlrn
-  !! Number of polaron bands
-  INTEGER :: nDOS_plrn
+  INTEGER :: start_band_plrn, end_band_plrn
+  !! Start and end band index in matrix element
+  INTEGER :: nstate_plrn
+  !! Number of polaron states calculated
+  INTEGER :: ndos_plrn
   !! Number of grid in polaron DOS calculation
-  INTEGER :: start_mode
-  !! start mode index
-  INTEGER :: cb_shift
-  !! CB shifted in polaron calculation
-  INTEGER :: diag_mode
-  !! diagonalization mode for polaron solver
-  INTEGER :: restart_polaron_mode
-  !! polaron restart mode
-  INTEGER :: polaron_type
+  INTEGER :: type_plrn
   !! polaron type (electron/hole)
-  INTEGER :: init_plrn_wf
+  INTEGER :: init_plrn
   !! initial polaron wavefuntion with 1:Gaussian package and 2:Random number
-  INTEGER :: niterPlrn
+  INTEGER :: niter_plrn
   !! Maximum number of polaron SCF loops.
-  REAL(KIND = DP) :: spherical_cutoff
-  !!  spherical_cutoff for fast convergence in polaron calculation
-  REAL(KIND = DP) :: conv_thr_polaron
+  INTEGER :: seed_plrn
+  !! The seed number to generate the random initial polaron wavefunction
+  INTEGER :: nhblock_plrn
+  !! The seed number to generate the random initial polaron wavefunction
+  INTEGER :: g_start_band_plrn, g_end_band_plrn
+  !! Start and end band in saving g matrix
+  INTEGER :: step_wf_grid_plrn
+  !! number of grid to skip in output of real space wavefunction
+  INTEGER :: io_lvl_plrn
+  !! number of grid to skip in output of real space wavefunction
+  INTEGER :: scell_mat(3,3)
+  !! Supercell transformation matrix for polaron
+  INTEGER :: init_ntau_plrn
+  !! Number of displacements to be read from file to calculate polaron energy landscape  
+  INTEGER :: nethrdg_plrn
+  !! Number of steps in the diagonalization if adaptive_ethrdg_plrn=.true.
+  REAL(KIND = DP) :: conv_thr_plrn
   !!  convergent threshold for polaron calculation
-  REAL(KIND = DP) :: r01, r02, r03
-  !!  x,y,z Carsteian coordinate of polaron centre
-  REAL(KIND = DP) :: emin_plrn, emax_plrn, sigma_edos_plrn
+  REAL(KIND = DP) :: edos_min_plrn, edos_max_plrn, edos_sigma_plrn
   !! Electron Energy range in polaron DOS calculation
-  REAL(KIND = DP) :: pmin_plrn, pmax_plrn, sigma_pdos_plrn
+  REAL(KIND = DP) :: pdos_min_plrn, pdos_max_plrn, pdos_sigma_plrn
   !! Phonon Energy range in polaron DOS calculation
-  REAL(KIND = DP) :: n_dop
-  !! extra added charge per cell (as tot_charge, with opposite sign)
-  REAL(KIND = DP) :: sigma_plrn
-  !! decay radius of polaron wavefunction in initialization
-  REAL(KIND = DP) :: ethr_Plrn
-  !! decay radius of polaron wavefunction in initialization
-  REAL(KIND = DP) :: mixing_Plrn
+  REAL(KIND = DP) :: init_sigma_plrn
+  !! decay radius of polaron wavefunction in Gaussian initialization
+  REAL(KIND = DP) :: init_k0_plrn(3)
+  !! center k of Gaussian initialization
+  REAL(KIND = DP) :: mixing_plrn
+  !! mixing weight in plrn iteration
+  REAL(KIND = DP) :: ethrdg_plrn
+  !! threshold in diagonalization of the polaron Hamiltonian
+  REAL(KIND = DP) :: beta_plrn(3)
   !! Mixing weight in Self-consistency
-  LOGICAL :: wfcelec
+  REAL(KIND = DP) :: r0_plrn(3)
+  !! Mixing weight in Self-consistency
+  REAL(KIND = DP) :: g_start_energy_plrn
+  !! Mixing weight in Self-consistency
+  REAL(KIND = DP) :: g_end_energy_plrn
+  !! Mixing weight in Self-consistency
+  REAL(KIND = DP) :: kappa_plrn
+  !! Mixing weight in Self-consistency
+  REAL(KIND = DP) :: g_power_order_plrn
+  !! kappa
+  REAL(KIND = DP) :: omega_LO_plrn
+  !! Higher optical phonon frequency
+  REAL(KIND = DP) :: m_eff_plrn
+  !! effective mass in LP model
+  REAL(KIND = DP) :: g_tol_plrn
+  !!
+  REAL(KIND = DP) :: as(3,3)
+  !! Supercell lattice vectors transformed by Smat, as=S*at
+  REAL(KIND = DP) :: bs(3,3)
+  !! Supercell reciprocal lattice vectors transformed by Sbar=transpose(inverse(Smat), bs=Sbar*bg  
+  REAL(KIND = DP) :: init_ethrdg_plrn
+  !! Initial coarse threshold to be used if adaptive_ethrdg_plrn=.true. 
+  !!
+  LOGICAL :: plrn
   !! if .true. calculates perturbated part of the wavefunction
-  LOGICAL :: restart_polaron
+  LOGICAL :: model_vertex_plrn
+  !! if .true. calculates vertex model
+  LOGICAL :: model_enband_plrn
+  !! if .true. calculates vertex model
+  LOGICAL :: model_phfreq_plrn
+  !! if .true. calculates vertex model
+  LOGICAL :: restart_plrn
   !! if .true. Using Ack from written outputs
-  LOGICAL :: model_vertex
-  !! if .true. Using model vertex and effective mass for polaron calculation
-  LOGICAL :: wfcelec_old
-  !! if .true. Using old algorithm by DS
   LOGICAL :: full_diagon_plrn
-   !! if .true. diagonalizing the polaron Hamiltonian with direct diagonalization
-  LOGICAL :: polaron_wf
+  !! if .true. diagonalizing the polaron Hamiltonian with direct diagonalization
+  LOGICAL :: cal_psir_plrn
   !! if .true. Generating a 3D-plot for polaron wavefunction
-  LOGICAL :: polaron_interpol
+  LOGICAL :: interp_Ank_plrn
   !! if .true. interpolating polaron A(k) from A(Re)
-  LOGICAL :: polaron_bq
+  LOGICAL :: interp_Bqu_plrn
   !! if .true. interpolating polaron phonon component bq from A(Re) and An(k)
-  LOGICAL :: polaron_dos
-  !! if .true. calculating polaron dos
-  LOGICAL :: electron_dos
-  !! if .true. calculating contributed electron dos
-  LOGICAL :: phonon_dos
-  !! if .true. calculating excited phonon dos
+  LOGICAL :: debug_plrn
+  !! if .true. interpolating polaron phonon component bq from A(Re) and An(k)
+  LOGICAL :: time_rev_A_plrn
+  !! if .true. time_rev_A_plrn
+  LOGICAL :: time_rev_U_plrn
+  !! if .true. time_rev_A_plrn
+  LOGICAL :: Mmn_plrn
+  !! if .true. time_rev_A_plrn
+  LOGICAL :: recal_Mmn_plrn
+  !! if .true. time_rev_A_plrn
+  LOGICAL :: scell_mat_plrn
+  !! if .true. activate supercell transformation for polaron
+  LOGICAL :: adapt_ethrdg_plrn
+  !! if .true. activate adaptive threshold in polaron davidson diagonalization
   ! ----------------------------------------------------------------------------------
   !
   !-----------------------------------------------------------------------
