@@ -2036,13 +2036,14 @@
         ENDDO ! ibnd
       ENDDO ! ik
       !
-      fmu = nstate - 2.d0 * gtemp(itemp) * fmu - nel
-      dmu = - 2.d0 * gtemp(itemp) * dmu
+      ! HP: factor 2 is already included above in wkfs(ik)
+      fmu = nstate - gtemp(itemp) * fmu - nel
+      dmu = - gtemp(itemp) * dmu
       !
       muout = muin - fmu / dmu
       !
       ! HP: linear mixing
-      muout = (1.0 - broyden_beta) * muin + broyden_beta * muout
+      muout = (1.d0 - ABS(broyden_beta)) * muin + ABS(broyden_beta) * muout
       !
       IF (ABS((muout - muin) / muin) <= eps6) conv = .TRUE.
       !
