@@ -5,6 +5,12 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
+! NOTE (Ivan Carnimeo, May, 05th, 2022): 
+!   cegterg and regterg have been ported to GPU with OpenACC, 
+!   the previous CUF versions (cegterg_gpu and regterg_gpu) have been removed, 
+!   and now cegterg and regterg are used for both CPU and GPU execution.
+!   If you want to see the previous code checkout to commit: df3080b231c5daf52295c23501fbcaa9bfc4bfcc (on Thu Apr 21 06:18:02 2022 +0000)
+!
 #define ZERO ( 0.D0, 0.D0 )
 #define ONE  ( 1.D0, 0.D0 )
 !
@@ -21,11 +27,15 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
   ! ... where H is an hermitean operator, e is a real scalar,
   ! ... S is an overlap matrix, evc is a complex vector
   !
+#if defined(__CUDA)
+  use cublas
+#endif
   USE util_param,    ONLY : DP
   USE mp_bands_util, ONLY : intra_bgrp_comm, inter_bgrp_comm, root_bgrp_id,&
                             nbgrp, my_bgrp_id, me_bgrp, root_bgrp
   USE mp,            ONLY : mp_sum, mp_gather, mp_bcast, mp_size,&
                             mp_type_create_column_section, mp_type_free
+  USE device_memcpy_m, ONLY : dev_memcpy, dev_memset
   !
   IMPLICIT NONE
   !
@@ -38,9 +48,6 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
     ! maximum dimension of the reduced basis set :
     !    (the basis set is refreshed when its dimension would exceed nvecx)
     ! umber of spin polarizations
-  INTEGER, PARAMETER :: blocksize = 256
-  INTEGER :: numblock
-    ! chunking parameters
   COMPLEX(DP), INTENT(INOUT) :: evc(npwx*npol,nvec)
     !  evc contains the  refined estimates of the eigenvectors  
   REAL(DP), INTENT(IN) :: ethr
@@ -77,12 +84,15 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
     ! defines a column section for communication
   INTEGER :: ierr
   COMPLEX(DP), ALLOCATABLE :: hc(:,:), sc(:,:), vc(:,:)
+  !$acc declare device_resident(hc, sc, vc)
     ! Hamiltonian on the reduced basis
     ! S matrix on the reduced basis
     ! the eigenvectors of the Hamiltonian
   REAL(DP), ALLOCATABLE :: ew(:)
+  !$acc declare device_resident(ew)
     ! eigenvalues of the reduced hamiltonian
   COMPLEX(DP), ALLOCATABLE :: psi(:,:), hpsi(:,:), spsi(:,:)
+  !$acc declare device_resident(psi, hpsi, spsi)
     ! work space, contains psi
     ! the product of H and psi
     ! the product of S and psi
@@ -92,8 +102,13 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
     ! threshold for empty bands
   INTEGER, ALLOCATABLE :: recv_counts(:), displs(:)
     ! receive counts and memory offsets
+  INTEGER, PARAMETER :: blocksize = 256
+  INTEGER :: numblock
+    ! chunking parameters
+  INTEGER :: i,j,k
   !
-  REAL(DP), EXTERNAL :: ddot
+  REAL(DP), EXTERNAL :: MYDDOT_VECTOR_GPU
+  !$acc routine(MYDDOT_VECTOR_GPU) vector
   !
   EXTERNAL  h_psi,    s_psi,    g_psi
     ! h_psi(npwx,npw,nvec,psi,hpsi)
@@ -108,6 +123,8 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
   nhpsi = 0
   CALL start_clock( 'cegterg' ); !write(*,*) 'start cegterg' ; FLUSH(6)
   !
+  !$acc data deviceptr(evc, e)
+  !
   IF ( nvec > nvecx / 2 ) CALL errore( 'cegterg', 'nvecx is too small', 1 )
   !
   ! ... threshold for empty bands
@@ -126,8 +143,10 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      !
   END IF
   !
+#if ! defined(__CUDA)
   ! compute the number of chuncks
   numblock  = (npw+blocksize-1)/blocksize
+#endif
   !
   ALLOCATE(  psi( npwx*npol, nvecx ), STAT=ierr )
   IF( ierr /= 0 ) &
@@ -163,7 +182,9 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
   nbase  = nvec
   conv   = .FALSE.
   !
-  CALL threaded_memcpy(psi, evc, nvec*npol*npwx*2)
+  !$acc host_data use_device(psi, hpsi, spsi, hc, sc)
+  CALL dev_memcpy(psi, evc, (/ 1 , npwx*npol /), 1, &
+                            (/ 1 , nvec /), 1)
   !
   ! ... hpsi contains h times the basis vectors
   !
@@ -185,7 +206,12 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
   if (n_start .le. n_end) &
   CALL ZGEMM( 'C','N', nbase, my_n, kdim, ONE, psi, kdmx, hpsi(1,n_start), kdmx, ZERO, hc(1,n_start), nvecx )
   !
-  if (n_start .le. n_end) CALL mp_sum( hc( 1:nbase, n_start:n_end ), intra_bgrp_comm )
+  if (n_start .le. n_end) & 
+#if defined(__CUDA)
+        CALL mp_sum( hc, 1, nbase, n_start, n_end , intra_bgrp_comm )
+#else
+        CALL mp_sum( hc(1:nbase, n_start:n_end), intra_bgrp_comm )
+#endif
   CALL mp_gather( hc, column_section_type, recv_counts, displs, root_bgrp_id, inter_bgrp_comm )
   !
   IF ( uspp ) THEN
@@ -193,7 +219,7 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      if (n_start .le. n_end) &
      CALL ZGEMM( 'C','N', nbase, my_n, kdim, ONE, psi, kdmx, spsi(1,n_start), kdmx, &
                  ZERO, sc(1,n_start), nvecx )
-     !     
+     !
   ELSE
      !
      if (n_start .le. n_end) &
@@ -202,11 +228,19 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      !
   END IF
   !
-  if (n_start .le. n_end) CALL mp_sum( sc( 1:nbase, n_start:n_end ), intra_bgrp_comm )
+  if (n_start .le. n_end) & 
+#if defined(__CUDA)
+         CALL mp_sum( sc, 1, nbase, n_start, n_end , intra_bgrp_comm )
+#else
+         CALL mp_sum( sc(1:nbase, n_start:n_end), intra_bgrp_comm )
+#endif
   CALL mp_gather( sc, column_section_type, recv_counts, displs, root_bgrp_id, inter_bgrp_comm )
+  !$acc end host_data
   !
   CALL mp_type_free( column_section_type )
   !
+  !$acc parallel vector_length(64) 
+  !$acc loop gang 
   DO n = 1, nbase
      !
      ! ... the diagonal of hc and sc must be strictly real
@@ -214,6 +248,7 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      hc(n,n) = CMPLX( REAL( hc(n,n) ), 0.D0 ,kind=DP)
      sc(n,n) = CMPLX( REAL( sc(n,n) ), 0.D0 ,kind=DP)
      !
+     !$acc loop vector 
      DO m = n + 1, nbase
         !
         hc(n,m) = CONJG( hc(m,n) )
@@ -222,13 +257,17 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      END DO
      !
   END DO
+  !$acc end parallel
   !
   CALL stop_clock( 'cegterg:init' )
   !
   IF ( lrot ) THEN
      !
-     vc(1:nbase,1:nbase) = ZERO
+     !$acc host_data use_device(vc)
+     CALL dev_memset(vc, ZERO, (/1, nbase/), 1, (/1, nbase/), 1)
+     !$acc end host_data
      !
+     !$acc parallel loop 
      DO n = 1, nbase
         !
         e(n) = REAL( hc(n,n) )
@@ -243,6 +282,7 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      !
      ! ... diagonalize the reduced hamiltonian
      !
+     !$acc host_data use_device(hc, sc, vc, ew)
      CALL start_clock( 'cegterg:diag' )
      IF( my_bgrp_id == root_bgrp_id ) THEN
         CALL diaghg( nbase, nvec, hc, sc, nvecx, ew, vc, me_bgrp, root_bgrp, intra_bgrp_comm )
@@ -253,7 +293,8 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      ENDIF
      CALL stop_clock( 'cegterg:diag' )
      !
-     e(1:nvec) = ew(1:nvec)
+     CALL dev_memcpy (e, ew, (/ 1, nvec /), 1 )
+     !$acc end host_data
      !
   END IF
   !
@@ -279,11 +320,18 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
            ! ... roots come first. This allows to use quick matrix-matrix 
            ! ... multiplications to set a new basis vector (see below)
            !
-           IF ( np /= n ) vc(:,np) = vc(:,n)
+           IF ( np /= n ) THEN
+             !$acc parallel loop 
+             DO i = 1, nvecx
+               vc(i,np) = vc(i,n)
+             END DO 
+           END IF 
            !
            ! ... for use in g_psi
            !
+           !$acc kernels 
            ew(nbase+np) = e(n)
+           !$acc end kernels 
            !
         END IF
         !
@@ -295,6 +343,8 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      !
      CALL divide(inter_bgrp_comm,nbase,n_start,n_end)
      my_n = n_end - n_start + 1; !write (*,*) nbase,n_start,n_end
+     !
+     !$acc host_data use_device(psi, spsi, vc)
      IF ( uspp ) THEN
         !
         if (n_start .le. n_end) &
@@ -308,8 +358,20 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
                     ZERO, psi(1,nb1), kdmx )
         !
      END IF
+     !$acc end host_data
 ! NB: must not call mp_sum over inter_bgrp_comm here because it is done later to the full correction
      !
+#if defined(__CUDA)
+     !$acc parallel loop collapse(3) 
+     DO np=1,notcnv
+        DO ipol = 1, npol
+           DO k=1,npwx
+             psi(k + (ipol-1)*npwx,nbase+np) = - ew(nbase+np)*psi(k + (ipol-1)*npwx,nbase+np)
+           END DO
+        END DO
+     END DO
+     !$acc end parallel 
+#else
      !$omp parallel do collapse(3)
      DO n = 1, notcnv
         DO ipol = 1, npol
@@ -323,21 +385,24 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         END DO
      END DO
      !$omp end parallel do
+#endif
      !
+     !$acc host_data use_device(psi, hpsi, vc, ew)
      if (n_start .le. n_end) &
      CALL ZGEMM( 'N','N', kdim, notcnv, my_n, ONE, hpsi(1,n_start), kdmx, vc(n_start,1), nvecx, &
                  ONE, psi(1,nb1), kdmx )
      CALL mp_sum( psi(:,nb1:nbase+notcnv), inter_bgrp_comm )
      !
      ! clean up garbage if there is any
-     IF (npw < npwx) psi(npw+1:npwx,nb1:nbase+notcnv) = ZERO
-     IF (npol == 2)  psi(npwx+npw+1:2*npwx,nb1:nbase+notcnv) = ZERO
+     IF (npw < npwx) CALL dev_memset(psi, ZERO, [npw+1,npwx], 1, [nb1, nbase+notcnv])
+     IF (npol == 2)  CALL dev_memset(psi, ZERO, [npwx+npw+1,2*npwx], 1, [nb1, nbase+notcnv])
      !
      CALL stop_clock( 'cegterg:update' )
      !
      ! ... approximate inverse iteration
      !
      CALL g_psi( npwx, npw, notcnv, npol, psi(1,nb1), ew(nb1) )
+     !$acc end host_data
      !
      ! ... "normalize" correction vectors psi(:,nb1:nbase+notcnv) in
      ! ... order to improve numerical stability of subspace diagonalization
@@ -345,25 +410,39 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      !
      ! ...         ew = <psi_i|psi_i>,  i = nbase + 1, nbase + notcnv
      !
+     !$acc parallel vector_length(96) 
+     !$acc loop gang private(nbn)
      DO n = 1, notcnv
         !
         nbn = nbase + n
         !
-        IF ( npol == 1 ) THEN
-           !
-           ew(n) = ddot( 2*npw, psi(1,nbn), 1, psi(1,nbn), 1 )
-           !
-        ELSE
-           !
-           ew(n) = ddot( 2*npw, psi(1,nbn), 1, psi(1,nbn), 1 ) + &
-                   ddot( 2*npw, psi(npwx+1,nbn), 1, psi(npwx+1,nbn), 1 )
-           !
-        END IF
+        ew(n) = MYDDOT_VECTOR_GPU( 2*npw, psi(1,nbn), psi(1,nbn) )
         !
      END DO
      !
-     CALL mp_sum( ew( 1:notcnv ), intra_bgrp_comm )
+     IF(npol.ne.1) THEN 
+       !$acc loop gang private(nbn)
+       DO n = 1, notcnv 
+         nbn = nbase + n
+         ew(n) = ew(n)  + MYDDOT_VECTOR_GPU( 2*npw, psi(npwx+1,nbn), psi(npwx+1,nbn) ) 
+       END DO 
+     END IF 
+     !$acc end parallel 
      !
+     !$acc host_data use_device(ew)
+     CALL mp_sum( ew( 1:notcnv ), intra_bgrp_comm )
+     !$acc end host_data
+     !
+#if defined(__CUDA)
+     !$acc parallel loop collapse(3) 
+     DO i = 1,notcnv
+        DO ipol = 1,npol
+           DO k=1,npw
+             psi(k + (ipol-1)*npwx,nbase+i) = psi(k+(ipol-1)*npwx,nbase+i)/SQRT( ew(i) )
+           END DO
+        END DO
+     END DO
+#else
      !$omp parallel do collapse(3)
      DO n = 1, notcnv
         DO ipol = 1, npol
@@ -377,9 +456,11 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         END DO
      END DO
      !$omp end parallel do
+#endif
      !
      ! ... here compute the hpsi and spsi of the new functions
      !
+     !$acc host_data use_device(psi, hpsi, spsi, hc, sc)
      CALL h_psi( npwx, npw, notcnv, psi(1,nb1), hpsi(1,nb1) ) ; nhpsi = nhpsi + notcnv
      !
      IF ( uspp ) CALL s_psi( npwx, npw, notcnv, psi(1,nb1), spsi(1,nb1) )
@@ -395,7 +476,12 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      CALL ZGEMM( 'C','N', notcnv, my_n, kdim, ONE, hpsi(1,nb1), kdmx, psi(1,n_start), kdmx, &
                  ZERO, hc(nb1,n_start), nvecx )
      !
-     if (n_start .le. n_end) CALL mp_sum( hc( nb1:nbase+notcnv, n_start:n_end ), intra_bgrp_comm )
+     if (n_start .le. n_end) &
+#if defined(__CUDA)
+       CALL mp_sum( hc, nb1, nbase+notcnv, n_start, n_end , intra_bgrp_comm )
+#else
+       CALL mp_sum( hc(nb1:nbase+notcnv, n_start:n_end) , intra_bgrp_comm )
+#endif
      CALL mp_gather( hc, column_section_type, recv_counts, displs, root_bgrp_id, inter_bgrp_comm )
      !
      CALL divide(inter_bgrp_comm,nbase+notcnv,n_start,n_end)
@@ -412,8 +498,14 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         !
      END IF
      !
-     if (n_start .le. n_end) CALL mp_sum( sc( nb1:nbase+notcnv, n_start:n_end ), intra_bgrp_comm )
+     if (n_start .le. n_end) & 
+#if defined(__CUDA)
+         CALL mp_sum( sc, nb1, nbase+notcnv, n_start, n_end , intra_bgrp_comm )
+#else
+         CALL mp_sum( sc(nb1:nbase+notcnv, n_start:n_end) , intra_bgrp_comm )
+#endif
      CALL mp_gather( sc, column_section_type, recv_counts, displs, root_bgrp_id, inter_bgrp_comm )
+     !$acc end host_data
      !
      CALL mp_type_free( column_section_type )
      !
@@ -421,6 +513,8 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
      !
      nbase = nbase + notcnv
      !
+     !$acc parallel vector_length(64)
+     !$acc loop gang
      DO n = 1, nbase
         !
         ! ... the diagonal of hc and sc must be strictly real
@@ -430,6 +524,7 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
            sc(n,n) = CMPLX( REAL( sc(n,n) ), 0.D0 ,kind=DP)
         ENDIF
         !
+        !$acc loop vector
         DO m = MAX(n+1,nb1), nbase
            !
            hc(n,m) = CONJG( hc(m,n) )
@@ -438,9 +533,11 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         END DO
         !
      END DO
+     !$acc end parallel 
      !
      ! ... diagonalize the reduced hamiltonian
      !
+     !$acc host_data use_device(hc, sc, vc, ew)
      CALL start_clock( 'cegterg:diag' )
      IF( my_bgrp_id == root_bgrp_id ) THEN
         CALL diaghg( nbase, nvec, hc, sc, nvecx, ew, vc, me_bgrp, root_bgrp, intra_bgrp_comm )
@@ -450,24 +547,27 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         CALL mp_bcast( ew, root_bgrp_id, inter_bgrp_comm )
      ENDIF
      CALL stop_clock( 'cegterg:diag' )
+     !$acc end host_data
      !
      ! ... test for convergence
      !
-     WHERE( btype(1:nvec) == 1 )
-        !
-        conv(1:nvec) = ( ( ABS( ew(1:nvec) - e(1:nvec) ) < ethr ) )
-        !
-     ELSEWHERE
-        !
-        conv(1:nvec) = ( ( ABS( ew(1:nvec) - e(1:nvec) ) < empty_ethr ) )
-        !
-     END WHERE
+     !$acc parallel loop copy(conv(1:nvec)) copyin(btype(1:nvec))
+     DO i = 1, nvec
+       IF(btype(i) == 1) THEN
+         conv(i) = ( ( ABS( ew(i) - e(i) ) < ethr ) )
+       ELSE
+         conv(i) = ( ( ABS( ew(i) - e(i) ) < empty_ethr ) )
+       END IF 
+     END DO 
+     !
      ! ... next line useful for band parallelization of exact exchange
      IF ( nbgrp > 1 ) CALL mp_bcast(conv,root_bgrp_id,inter_bgrp_comm)
      !
      notcnv = COUNT( .NOT. conv(:) )
      !
-     e(1:nvec) = ew(1:nvec)
+     !$acc host_data use_device(ew)
+     CALL dev_memcpy (e, ew, (/ 1, nvec /) )
+     !$acc end host_data
      !
      ! ... if overall convergence has been achieved, or the dimension of
      ! ... the reduced basis set is becoming too large, or in any case if
@@ -482,8 +582,10 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         !
         CALL divide(inter_bgrp_comm,nbase,n_start,n_end)
         my_n = n_end - n_start + 1; !write (*,*) nbase,n_start,n_end
+        !$acc host_data use_device(psi, vc)
         CALL ZGEMM( 'N','N', kdim, nvec, my_n, ONE, psi(1,n_start), kdmx, vc(n_start,1), nvecx, &
                     ZERO, evc, kdmx )
+        !$acc end host_data
         CALL mp_sum( evc, inter_bgrp_comm )
         !
         IF ( notcnv == 0 ) THEN
@@ -509,38 +611,54 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
         !
         ! ... refresh psi, H*psi and S*psi
         !
-        CALL threaded_memcpy(psi, evc, nvec*npol*npwx*2)
+        !$acc host_data use_device(psi, hpsi, spsi, vc)
+        CALL dev_memcpy(psi, evc, (/ 1, npwx*npol /), 1, &
+                                      (/ 1, nvec /), 1)
         !
         IF ( uspp ) THEN
            !
            CALL ZGEMM( 'N','N', kdim, nvec, my_n, ONE, spsi(1,n_start), kdmx, vc(n_start,1), nvecx, &
                        ZERO, psi(1,nvec+1), kdmx)
-           CALL threaded_memcpy(spsi, psi(1,nvec+1), nvec*npol*npwx*2)
+           CALL dev_memcpy(spsi, psi(:,nvec+1:), &
+                                        (/1, npwx*npol/), 1, &
+                                        (/1, nvec/), 1)
            CALL mp_sum( spsi(:,1:nvec), inter_bgrp_comm )
            !
         END IF
         !
         CALL ZGEMM( 'N','N', kdim, nvec, my_n, ONE, hpsi(1,n_start), kdmx, vc(n_start,1), nvecx, &
                     ZERO, psi(1,nvec+1), kdmx )
-        CALL threaded_memcpy(hpsi, psi(1,nvec+1), nvec*npol*npwx*2)
+        CALL dev_memcpy(hpsi, psi(:,nvec+1:), &
+                                        (/1, npwx*npol/), 1, &
+                                        (/1, nvec/), 1)
         CALL mp_sum( hpsi(:,1:nvec), inter_bgrp_comm )
+        !$acc end host_data
         !
         ! ... refresh the reduced hamiltonian 
         !
         nbase = nvec
         !
-        hc(1:nbase,1:nbase) = ZERO
-        sc(1:nbase,1:nbase) = ZERO
-        vc(1:nbase,1:nbase) = ZERO
+        ! These variables are set to ZERO in the CUF Kernel below
+        !hc(1:nbase,1:nbase) = ZERO
+        !sc(1:nbase,1:nbase) = ZERO
+        !vc(1:nbase,1:nbase) = ZERO
         !
+        !$acc kernels 
         DO n = 1, nbase
-           !
            hc(n,n) = CMPLX( e(n), 0.0_DP ,kind=DP)
-           !
            sc(n,n) = ONE
            vc(n,n) = ONE
+           DO j = n+1, nbase
+              hc(j,n) = ZERO
+              hc(n,j) = ZERO
+              sc(j,n) = ZERO
+              sc(n,j) = ZERO
+              vc(j,n) = ZERO
+              vc(n,j) = ZERO
+           END DO
            !
         END DO
+        !$acc end kernels
         !
         CALL stop_clock( 'cegterg:last' )
         !
@@ -561,6 +679,8 @@ SUBROUTINE cegterg( h_psi, s_psi, uspp, g_psi, &
   DEALLOCATE( hpsi )
   DEALLOCATE( psi )
   !
+  !$acc end data 
+  !
   CALL stop_clock( 'cegterg' ); !write(*,*) 'stop cegterg' ; FLUSH(6)
   !call print_clock( 'cegterg' )
   !call print_clock( 'cegterg:init' )
