@@ -1,4 +1,4 @@
-# LaTeX2HTML 2024 (Released January 1, 2024)
+# LaTeX2HTML 2021.2 (Released July 1, 2021)
 # Associate labels original text with physical files.
 
 
@@ -13,9 +13,17 @@ $noresave{$key} = "$nosave";
 1;
 
 
-# LaTeX2HTML 2024 (Released January 1, 2024)
+# LaTeX2HTML 2021.2 (Released July 1, 2021)
 # labels from external_latex_labels array.
 
 
+$key = q/Sec:para/;
+$external_latex_labels{$key} = q|4 Parallelism|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Examples/;
+$external_latex_labels{$key} = q|3.1 Running examples|; 
+$noresave{$key} = "$nosave";
+
 1;
 
