@@ -10,7 +10,7 @@
 SUBROUTINE phq_readin()
   !----------------------------------------------------------------------------
   !! This routine reads the control variables for the program \(\texttt{phononq}\).
-  !! A second routine, \(\texttt{read_file}\), reads the variables saved to file
+  !! A second routine, \(\texttt{read_file_ph}\), reads the variables saved to file
   !! by the self-consistent program.
   !
   !
@@ -28,7 +28,6 @@ SUBROUTINE phq_readin()
   USE uspp,          ONLY : okvan
   USE fixed_occ,     ONLY : tfixed_occ
   USE lsda_mod,      ONLY : lsda, nspin
-  USE fft_base,      ONLY : dffts
   USE cellmd,        ONLY : lmovecell
   USE run_info,      ONLY : title
   USE control_ph,    ONLY : maxter, alpha_mix, lgamma_gamma, epsil, &
@@ -39,7 +38,8 @@ SUBROUTINE phq_readin()
                             ext_recover, ext_restart, u_from_file, ldiag, &
                             search_sym, lqdir, electron_phonon, tmp_dir_phq, &
                             rec_code_read, qplot, only_init, only_wfc, &
-                            low_directory_check, nk1, nk2, nk3, k1, k2, k3
+                            low_directory_check, nk1, nk2, nk3, k1, k2, k3, &
+                            dftd3_hess
   USE save_ph,       ONLY : tmp_dir_save, save_ph_input_variables
   USE gamma_gamma,   ONLY : asr
   USE partial,       ONLY : atomo, nat_todo, nat_todo_input
@@ -108,6 +108,7 @@ SUBROUTINE phq_readin()
   INTEGER, ALLOCATABLE :: wqaux(:)
   INTEGER :: nqaux, iq
   CHARACTER(len=80) :: diagonalization='david'
+  LOGICAL :: needwf_ph=.TRUE.
   !
   NAMELIST / INPUTPH / tr2_ph, amass, alpha_mix, niter_ph, nmix_ph,  &
                        nat_todo, verbosity, iverbosity, outdir, epsil,  &
@@ -125,7 +126,7 @@ SUBROUTINE phq_readin()
                        lshift_q, read_dns_bare, d2ns_type, diagonalization, &
                        ldvscf_interpolate, do_long_range, do_charge_neutral, &
                        wpot_dir, ahc_dir, ahc_nbnd, ahc_nbndskip, &
-                       skip_upperfan
+                       skip_upperfan, dftd3_hess
 
   ! tr2_ph       : convergence threshold
   ! amass        : atomic masses
@@ -209,6 +210,8 @@ SUBROUTINE phq_readin()
   ! ahc_nbndskip: Number of bands to exclude when computing the self-energy.
   ! skip_upperfan: If .true., skip the calculation of upper Fan self-energy.
   !
+  ! dftd3_hess: file from where the dftd3 hessian is read
+  !
   ! Note: meta_ionode is a single processor that reads the input
   !       (ionode is also a single processor but per image)
   !       Data read from input is subsequently broadcast to all processors
@@ -277,6 +280,7 @@ SUBROUTINE phq_readin()
   fildyn       = 'matdyn'
   fildrho      = ' '
   fildvscf     = ' '
+  dftd3_hess   = ' '
   ldisp        = .FALSE.
   nq1          = 0
   nq2          = 0
@@ -714,9 +718,12 @@ SUBROUTINE phq_readin()
   !
   IF (reduce_io) io_level=0
   !
-  ! DFPT+U: the occupation matrix ns is read via read_file
+  ! Here all needed data from the scf calculation are read
   !
-  CALL read_file ( )
+  ! IO of wfcs in collected format is not needed for recover run
+  needwf_ph = .NOT. recover
+  !
+  CALL read_file_ph ( needwf_ph )
   !
   magnetic_sym=noncolin .AND. domag
   !
@@ -769,8 +776,8 @@ SUBROUTINE phq_readin()
   IF (ts_vdw) CALL errore('phq_readin',&
      'The phonon code with TS-VdW is not yet available',1)
 
-  IF (ldftd3) CALL errore('phq_readin',&
-     'The phonon code with Grimme''s DFT-D3 is not yet available',1)
+  !IF (ldftd3) CALL errore('phq_readin',&
+  !   'The phonon code with Grimme''s DFT-D3 is not yet available',1)
 
   IF ( xclib_dft_is('meta') ) CALL errore('phq_readin',&
      'The phonon code with meta-GGA functionals is not yet available',1)
@@ -778,9 +785,6 @@ SUBROUTINE phq_readin()
   IF ( xclib_dft_is('hybrid') ) CALL errore('phq_readin',&
      'The phonon code with hybrid functionals is not yet available',1)
 
-  IF (okpaw.and.(lraman.or.elop)) CALL errore('phq_readin',&
-     'The phonon code with paw and raman or elop is not yet available',1)
-
   IF (magnetic_sym) THEN
 
      WRITE(stdout,'(/5x,a)') "Phonon calculation in the non-collinear magnetic case;"
@@ -788,16 +792,19 @@ SUBROUTINE phq_readin()
      WRITE(stdout,'(5x,a)')  "045115 (2019) for the theoretical background."
 
      IF (okpaw) CALL errore('phq_readin',&
-          'The phonon code with paw and domag is not available yet',1)
+          'The phonon code with paw and magnetism is not yet available',1)
   ENDIF
 
+  IF (okpaw.and.(lraman.or.elop)) CALL errore('phq_readin',&
+     'The phonon code with paw and raman or elop is not yet available',1)
+
   IF (okvan.and.(lraman.or.elop)) CALL errore('phq_readin',&
      'The phonon code with US-PP and raman or elop not yet available',1)
 
   IF (noncolin.and.(lraman.or.elop)) CALL errore('phq_readin', &
       'lraman, elop, and noncolin not programmed',1)
-  IF ( (noncolin.or.lspinorb) .and. elph ) CALL errore('phq_readin', &
-      'el-ph coefficient calculation disabled in noncolinear/spinorbit case',1)
+  IF ( domag .and. lspinorb .and. elph ) CALL errore('phq_readin', & 
+    'el-ph coefficient calculation disabled in magnetic spinorbit case',1)
 
   IF (lmovecell) CALL errore('phq_readin', &
       'The phonon code is not working after vc-relax',1)
@@ -808,6 +815,9 @@ SUBROUTINE phq_readin()
   IF(elph_mat.and.npool.ne.1) call errore('phq_readin',&
        'el-ph with wannier : pools not implemented',1)
 
+  IF (elph .AND. okpaw) CALL errore('phq_readin',&
+     'Electron-phonon calculations with PAW not tested',1)
+
   IF(elph.and.nimage>1) call errore('phq_readin',&
        'el-ph with images not implemented',1)
 
@@ -938,6 +948,12 @@ SUBROUTINE phq_readin()
                   (nq1 .LE. 0 .OR. nq2 .LE. 0 .OR. nq3 .LE. 0)) &
        CALL errore('phq_readin','nq1, nq2, and nq3 must be greater than 0',1)
 
+  !
+  ! If dftd3_hess is not specified, use a default name set from prefix
+  !
+  IF ( dftd3_hess == ' ' ) dftd3_hess = trim(prefix)//'.hess'
+  dftd3_hess = TRIM(tmp_dir)//TRIM(dftd3_hess)
+
   CALL save_ph_input_variables()
   !
   RETURN
