@@ -45,6 +45,9 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
   USE wvfct,                ONLY : nbnd, npwx
   USE scf,                  ONLY : rho, vrs
+#if defined(__CUDA)
+  USE scf_gpum,             ONLY : vrs_d
+#endif
   USE uspp,                 ONLY : okvan, vkb, deeq_nc
   USE uspp_param,           ONLY : nhm
   USE noncollin_module,     ONLY : noncolin, domag, npol, nspin_mag
@@ -87,7 +90,6 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE response_kernels,     ONLY : sternheimer_kernel
   USE uspp_init,            ONLY : init_us_2
   USE sym_def_module,       ONLY : sym_def
-
   implicit none
 
   integer :: irr
@@ -150,6 +152,8 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   integer  :: iq_dummy
   real(DP) :: tcpu, get_clock ! timing variables
   character(len=256) :: filename
+
+  integer :: nnr
   !
   IF (rec_code_read > 20 ) RETURN
 
@@ -161,12 +165,15 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   IF (noncolin.AND.domag) nsolv=2
 
   allocate (dvscfin ( dfftp%nnr , nspin_mag , npe))
+  nnr = dfftp%nnr
   dvscfin=(0.0_DP,0.0_DP)
   if (doublegrid) then
      allocate (dvscfins (dffts%nnr , nspin_mag , npe))
+     nnr = dffts%nnr
   else
      dvscfins => dvscfin
   endif
+  !$acc enter data create(dvscfins(1:nnr, 1:nspin_mag, 1:npe))
   allocate (drhoscfh ( dfftp%nnr, nspin_mag , npe))
   allocate (dvscfout ( dfftp%nnr, nspin_mag , npe))
   allocate (dbecsum ( (nhm * (nhm + 1))/2 , nat , nspin_mag , npe))
@@ -273,16 +280,18 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
               ! Hubbard potential dvbare_hub_q * psi_kpoint
               ! is calculated and added to dvpsi.
               !
-              IF (lda_plus_u) CALL dvqhub_barepsi_us(ik, u(1, mode))
+              IF (lda_plus_u) CALL dvqhub_barepsi_us(ik, u(1,mode))
               !
            ELSE
               IF (okvan) THEN
                  deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,2)
+                 !$acc update device(deeq_nc)
                  int1_nc(:,:,:,:,:) = int1_nc_save(:,:,:,:,:,2)
               ENDIF
               CALL dvqpsi_us(ik, u(1, mode), .FAlSE., becpt, alphapt)
               IF (okvan) THEN
                  deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,1)
+                 !$acc update device(deeq_nc)
                  int1_nc(:,:,:,:,:) = int1_nc_save(:,:,:,:,:,1)
               ENDIF
            ENDIF
@@ -322,7 +331,13 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
               IF (okvan) int3_nc(:,:,:,:,:) = int3_save(:,:,:,:,:,2)
            ENDIF
            vrs(:, 2:4) = -vrs(:, 2:4)
-           IF (okvan) deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,2)
+#if defined(__CUDA)
+           vrs_d = vrs
+#endif
+           IF (okvan) THEN
+                   deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,2)
+                   !$acc update device(deeq_nc)
+           ENDIF
         ENDIF
         !
         ! set threshold for iterative solution of the linear system
@@ -347,7 +362,13 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
               IF (okvan) int3_nc(:,:,:,:,:) = int3_save(:,:,:,:,:,1)
            ENDIF
            vrs(:, 2:4) = -vrs(:, 2:4)
-           IF (okvan) deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,1)
+#if defined(__CUDA)
+           vrs_d = vrs
+#endif
+           IF (okvan) THEN
+                   deeq_nc(:,:,:,:) = deeq_nc_save(:,:,:,:,1)
+                   !$acc update device(deeq_nc)
+           ENDIF
         ENDIF
         !
      END DO ! isolv
@@ -593,6 +614,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   IF (noncolin) deallocate (dbecsum_nc)
   deallocate (dvscfout)
   deallocate (drhoscfh)
+  !$acc exit data delete(dvscfins)
   if (doublegrid) deallocate (dvscfins)
   deallocate (dvscfin)
   deallocate(aux2)
