@@ -93,7 +93,9 @@ SUBROUTINE iosys()
                             smearing_          => smearing, &
                             degauss_           => degauss, &
                             tot_charge_        => tot_charge, &
-                            tot_magnetization_ => tot_magnetization
+                            tot_magnetization_ => tot_magnetization, &
+                            degauss_cond_      => degauss_cond, &
+                            nelec_cond_        => nelec_cond
   USE ktetra,        ONLY : tetra_type
   USE start_k,       ONLY : init_start_k
   !
@@ -152,12 +154,18 @@ SUBROUTINE iosys()
                             lsda
   !
   USE relax,         ONLY : epse, epsf, epsp, starting_scf_threshold
+  !
+  USE control_flags, ONLY : sic, scissor
+  USE sic_mod,       ONLY : pol_type_ => pol_type, sic_gamma_ => sic_gamma, &
+                            sic_energy_ => sic_energy
+  USE sci_mod,       ONLY : sci_vb_ => sci_vb, sci_cb_ => sci_cb
+ 
   !
   USE extrapolation, ONLY : pot_order, wfc_order
   USE control_flags, ONLY : isolve, max_cg_iter, max_ppcg_iter, david, &
                             rmm_ndim, rmm_conv, gs_nblock, rmm_with_davidson, &
                             tr2, imix, gamma_only, &
-                            nmix, iverbosity, smallmem, niter, &
+                            nmix, iverbosity, smallmem, nexxiter, niter, &
                             io_level, ethr, lscf, lbfgs, lmd, &
                             lbands, lconstrain, restart, &
                             llondon, ldftd3, do_makov_payne, lxdm, &
@@ -182,7 +190,11 @@ SUBROUTINE iosys()
                             max_xml_steps_    => max_xml_steps 
   USE check_stop,    ONLY : max_seconds_ => max_seconds
   !
-  USE wvfct,         ONLY : nbnd_ => nbnd
+  USE wvfct,         ONLY : nbnd_ => nbnd, &
+                            nbnd_cond_ => nbnd_cond              
+  !
+  USE two_chem,      ONLY : twochem_ => twochem
+  !  
   USE gvecw,         ONLY : ecfixed_ => ecfixed, &
                             qcutz_   => qcutz, &
                             q2sigma_ => q2sigma
@@ -235,7 +247,8 @@ SUBROUTINE iosys()
                                gdir, nppstr, wf_collect,lelfield,lorbm,efield, &
                                nberrycyc, efield_cart, lecrpa,                 &
                                lfcp, vdw_table_name, memory, max_seconds,      &
-                               tqmmm, efield_phase, gate, max_xml_steps, trism
+                               tqmmm, efield_phase, gate, max_xml_steps,       &
+                               trism, twochem
 
   !
   ! ... SYSTEM namelist
@@ -254,13 +267,14 @@ SUBROUTINE iosys()
                                input_dft, la2F, starting_ns_eigenvalue,     &
                                x_gamma_extrapolation, nqx1, nqx2, nqx3,     &
                                exxdiv_treatment, yukawa, ecutvcut,          &
+                               pol_type, sic_gamma, sic_energy, sci_vb, sci_cb, &
                                exx_fraction, screening_parameter, ecutfock, &
                                gau_parameter, localization_thr, scdm, ace,  &
                                scdmden, scdmgrd, nscdm, n_proj,             & 
                                edir, emaxpos, eopreg, eamp, noncolin, lambda, &
                                angle1, angle2, constrained_magnetization,     &
                                B_field, fixed_magnetization, report, lspinorb,&
-                               starting_spin_angle, assume_isolated,spline_ps,&
+                               starting_spin_angle, assume_isolated,        &
                                vdw_corr, london, london_s6, london_rcut, london_c6, &
                                london_rvdw, dftd3_threebody, dftd3_version,   &
                                ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,     &
@@ -270,11 +284,12 @@ SUBROUTINE iosys()
                                esm_bc, esm_efield, esm_w, esm_nfit, esm_a,    &
                                lgcscf,                                        &
                                zgate, relaxz, block, block_1, block_2,        &
-                               block_height, lgcscf
+                               block_height, lgcscf, nbnd_cond, nelec_cond,   &
+                               degauss_cond
   !
   ! ... ELECTRONS namelist
   !
-  USE input_parameters, ONLY : electron_maxstep, mixing_mode, mixing_beta, &
+  USE input_parameters, ONLY : exx_maxstep, electron_maxstep, mixing_mode, mixing_beta, &
                                mixing_ndim, mixing_fixed_ns, conv_thr,     &
                                tqr, tq_smoothing, tbeta_smoothing,         &
                                diago_thr_init,                             &
@@ -329,7 +344,6 @@ SUBROUTINE iosys()
   USE dftd3_qe,              ONLY : dftd3_xc, dftd3, dftd3_in
   USE xdm_module,            ONLY : init_xdm, a1i, a2i
   USE tsvdw_module,          ONLY : vdw_isolated, vdw_econv_thr
-  USE uspp_data,             ONLY : spline_ps_ => spline_ps
   !
   USE qexsd_input,           ONLY : qexsd_input_obj
   USE qes_types_module,      ONLY : input_type
@@ -337,6 +351,11 @@ SUBROUTINE iosys()
 #if defined (__ENVIRON)
   USE plugin_flags,          ONLY : use_environ
   USE environ_base_module,   ONLY : read_environ_input, init_environ_setup
+#endif
+#if defined (__OSCDFT)
+  USE plugin_flags,          ONLY : use_oscdft
+  USE oscdft_base,           ONLY : oscdft_ctx
+  USE oscdft_input,          ONLY : oscdft_read_input
 #endif
   !
   IMPLICIT NONE
@@ -353,7 +372,7 @@ SUBROUTINE iosys()
   CHARACTER(LEN=256):: dft_
   !
   INTEGER  :: ia, nt, tempunit, i, j, ibrav_mp
-  LOGICAL  :: exst, parallelfs, domag, stop_on_error, is_tau_read
+  LOGICAL  :: exst, parallelfs, domag, stop_on_error, is_tau_read, sm_wasnt_set
   REAL(DP) :: at_dum(3,3), theta, phi, ecutwfc_pp, ecutrho_pp, V
   CHARACTER(len=256) :: tempfile
   INTEGER, EXTERNAL :: at2ibrav
@@ -563,6 +582,15 @@ SUBROUTINE iosys()
   nstep_ = nstep
   tstress_ = lmovecell .OR. ( tstress .and. lscf )
   !
+  sic_gamma_ = sic_gamma
+  sic_energy_ = sic_energy
+  IF(sic_gamma /= 0.d0 ) sic = .true.
+  pol_type_ = trim(pol_type)
+  sci_vb_ = sci_vb
+  sci_cb_ = sci_cb
+  IF(sci_vb .NE. 0.d0 .or. sci_cb .NE. 0.d0 ) scissor = .true.
+  IF(scissor .and. nspin .ne. 2) CALL errore('allocate_scissor', 'spin polarized calculation required',1)
+  !
   ! ELECTRIC FIELDS (SAWTOOTH), GATE FIELDS
   !
   IF ( tefield .and. ( .not. nosym ) .and. ( .not. gate ) ) THEN
@@ -651,6 +679,8 @@ SUBROUTINE iosys()
   !
   degauss_ = degauss
   smearing_ = smearing
+  degauss_cond_ = degauss_cond
+  nelec_cond_ = nelec_cond
   !
   IF( ltetra ) THEN
      IF( lforce ) CALL infomsg( 'iosys', &
@@ -660,6 +690,7 @@ SUBROUTINE iosys()
   END IF
   IF( nbnd < 1 ) CALL errore( 'iosys', 'nbnd less than 1', nbnd ) 
   nbnd_    = nbnd
+  nbnd_cond_ = nbnd_cond
   !
   two_fermi_energies = ( tot_magnetization /= -10000._DP)
   IF ( two_fermi_energies .and. tot_magnetization < -9999._DP) &
@@ -727,40 +758,43 @@ SUBROUTINE iosys()
   lspinorb_ = lspinorb
   lforcet_ = lforcet
   !
+  ! ... starting_magnetization(nt) = sm_not_set means "not set"
+  ! ... take notice and set to the default (zero)
+  !
+  sm_wasnt_set = ALL (starting_magnetization(1:ntyp) == sm_not_set)
+  DO nt = 1, ntyp
+     IF ( starting_magnetization(nt) == sm_not_set ) &
+          starting_magnetization(nt) = 0.0_dp
+  END DO
+  !
   SELECT CASE( trim( constrained_magnetization ) )
   CASE( 'none' )
      !
-     ! ... starting_magnetization(nt) = sm_not_set means "not set"
      ! ... if no constraints are imposed on the magnetization, 
      ! ... starting_magnetization must be set for at least one atomic type
      !
      IF ( lscf .AND. lsda .AND. ( .NOT. tfixed_occ ) .AND. &
-          ( .not. two_fermi_energies )  .AND. &
-          ALL (starting_magnetization(1:ntyp) == sm_not_set) ) &
+          ( .not. two_fermi_energies )  .AND. sm_wasnt_set ) &
         CALL errore('iosys','some starting_magnetization MUST be set', 1 )
      !
      ! ... bring starting_magnetization between -1 and 1
      !
      DO nt = 1, ntyp
-        !
-        IF ( starting_magnetization(nt) == sm_not_set ) THEN
-           starting_magnetization(nt) = 0.0_dp
-        ELSEIF ( starting_magnetization(nt) > 1.0_dp ) THEN
-          starting_magnetization(nt) = 1.0_dp
-        ELSEIF ( starting_magnetization(nt) <-1.0_dp ) THEN
-          starting_magnetization(nt) =-1.0_dp
-        ENDIF
-        !
+        starting_magnetization(nt) = MIN( 1.0_dp,starting_magnetization(nt))
+        starting_magnetization(nt) = MAX(-1.0_dp,starting_magnetization(nt))
      ENDDO
      !
      i_cons = 0
      !
   CASE( 'atomic' )
+     !
+     ! ... if "atomic" constraints are imposed on the magnetization, 
+     ! ... starting_magnetization must be set for at least one atomic type
      !
      IF ( nspin == 1 ) &
         CALL errore( 'iosys','constrained atomic magnetizations ' // &
                    & 'require nspin=2 or 4 ', 1 )
-     IF ( ALL (starting_magnetization(1:ntyp) == sm_not_set) ) &
+     IF ( sm_wasnt_set ) &
         CALL errore( 'iosys','constrained atomic magnetizations ' // &
                    & 'require that some starting_magnetization is set', 1 )
      !
@@ -1040,6 +1074,7 @@ SUBROUTINE iosys()
   !
   ethr = diago_thr_init
   tr2   = conv_thr
+  nexxiter = exx_maxstep
   niter = electron_maxstep
   adapt_thr = adaptive_thr
   tr2_init  = conv_thr_init
@@ -1241,6 +1276,7 @@ SUBROUTINE iosys()
   efield_     = efield
   nberrycyc_  = nberrycyc
   efield_cart_ = efield_cart
+  twochem_    = twochem
   SELECT CASE(efield_phase)
      CASE( 'none' )
         phase_control=0
@@ -1325,7 +1361,6 @@ SUBROUTINE iosys()
   !
   ! MISCELLANEOUS VARIABLES
   !
-  spline_ps_ = spline_ps
   la2F_      = la2F
   max_seconds_ = max_seconds
   !
@@ -1474,11 +1509,19 @@ SUBROUTINE iosys()
   !
   ! ... once input variables have been stored, read optional plugin input files
   !
+#if defined(__LEGACY_PLUGINS)
+  CALL plugin_read_input('PW')
+#endif 
 #if defined (__ENVIRON)
   IF (use_environ) THEN
      CALL read_environ_input()
      CALL init_environ_setup('PW')
   END IF
+#endif
+#if defined (__OSCDFT)
+  IF (use_oscdft) THEN
+     CALL oscdft_read_input(oscdft_ctx%inp)
+  END IF
 #endif
   !
   ! ... Files (for compatibility) and directories
