@@ -45,6 +45,11 @@ SUBROUTINE mix_rho( input_rhout, rhoin, alphamix, dr2, tr2_min, iter, n_iter,&
   USE ldaU,           ONLY : lda_plus_u, lda_plus_u_kind, ldim_u, &
                              max_num_neighbors, nsg, nsgnew
   USE buffers,        ONLY : open_buffer, close_buffer, get_buffer, save_buffer
+#if defined (__OSCDFT)
+  USE plugin_flags,     ONLY : use_oscdft
+  USE oscdft_base,      ONLY : oscdft_ctx
+  USE oscdft_functions, ONLY : oscdft_mix_rho
+#endif
   !
   IMPLICIT NONE
   !
@@ -162,6 +167,9 @@ SUBROUTINE mix_rho( input_rhout, rhoin, alphamix, dr2, tr2_min, iter, n_iter,&
      conv = conv .AND. ( ABS( ef - gcscf_mu ) < gcscf_eps )
      !
   END IF
+#if defined(__OSCDFT)
+  IF (use_oscdft) CALL oscdft_mix_rho(oscdft_ctx, conv)
+#endif
   !
   IF ( conv .OR. dr2 < tr2_min ) THEN
      !
@@ -446,7 +454,6 @@ SUBROUTINE approx_screening( drho )
   USE klist,         ONLY : nelec
   USE control_flags, ONLY : ngm0
   USE scf,           ONLY : mix_type
-  USE wavefunctions, ONLY : psic
   USE gcscf_module,  ONLY : lgcscf, gcscf_gk
   !
   IMPLICIT NONE
@@ -484,14 +491,13 @@ SUBROUTINE approx_screening2( drho, rhobest )
   USE constants,            ONLY : e2, pi, tpi, fpi, eps8, eps32
   USE cell_base,            ONLY : omega, tpiba2
   USE gvect,                ONLY : gg, ngm
-  USE wavefunctions, ONLY : psic
   USE klist,                ONLY : nelec
   USE control_flags,        ONLY : ngm0, gamma_only
   USE scf,                  ONLY : mix_type, local_tf_ddot
   USE mp,                   ONLY : mp_sum
   USE mp_bands,             ONLY : intra_bgrp_comm
   USE fft_base,             ONLY : dffts
-  USE fft_interfaces,       ONLY : fwfft, invfft
+  USE fft_rho,              ONLY : rho_r2g, rho_g2r
   USE gcscf_module,         ONLY : lgcscf, gcscf_gk, gcscf_gh
   !
   IMPLICIT NONE
@@ -512,9 +518,11 @@ SUBROUTINE approx_screening2( drho, rhobest )
     w(:,:),     &! w(ngm0,mmx)
     dv(:),      &! dv(ngm0)
     vbest(:),   &! vbest(ngm0)
-    wbest(:)     ! wbest(ngm0)
+    wbest(:),   &! wbest(ngm0)
+    auxg(:,:)    ! auxg(dffts%nnr,1)
   REAL(DP), ALLOCATABLE :: &
-    alpha(:)     ! alpha(dffts%nnr)
+    alpha(:),   &! alpha(dffts%nnr)
+    auxr(:)      ! auxr(dffts%nnr)
   !
   INTEGER             :: ir, ig
   REAL(DP), PARAMETER :: one_third = 1.D0 / 3.D0
@@ -523,30 +531,18 @@ SUBROUTINE approx_screening2( drho, rhobest )
   !
   IF ( (.NOT. lgcscf) .AND. gg(1) < eps8 ) drho%of_g(1,1) = ZERO
   !
+  ALLOCATE( auxr(dffts%nnr), auxg(dffts%nnr,1) )
   ALLOCATE( alpha( dffts%nnr ) )
   ALLOCATE( v( ngm0, mmx ), &
             w( ngm0, mmx ), dv( ngm0 ), vbest( ngm0 ), wbest( ngm0 ) )
   !
-  !$omp parallel
-     !
-     CALL threaded_barrier_memset(psic, 0.0_DP, dffts%nnr*2)
-     !$omp do
-     DO ig = 1, ngm0
-        psic(dffts%nl(ig)) = rhobest%of_g(ig,1)
-     ENDDO
-     !$omp end do nowait
-     !
-  !$omp end parallel
-  !
-  ! ... calculate alpha from density
-  !
-  CALL invfft ('Rho', psic, dffts)
+  CALL rho_g2r( dffts, rhobest%of_g(:,1), auxr )
   !
   avg_rsm1 = 0.D0
   !
   !$omp parallel do reduction(+:avg_rsm1)
   DO ir = 1, dffts%nnr
-     alpha(ir) = ABS( REAL( psic(ir) ) )
+     alpha(ir) = ABS( auxr(ir) )
      !
      IF ( alpha(ir) > eps32 ) THEN
         !
@@ -573,39 +569,22 @@ SUBROUTINE approx_screening2( drho, rhobest )
   !
   ! ... calculate deltaV and the first correction vector
   !
-  !$omp parallel
-     CALL threaded_barrier_memset(psic, 0.0_DP, dffts%nnr*2)
-     !$omp do
-     DO ig = 1, ngm0
-        psic(dffts%nl(ig)) = drho%of_g(ig,1)
-     ENDDO
-     !$omp end do nowait
-     !
-     IF ( gamma_only ) THEN
-        !$omp do
-        DO ig = 1, ngm0
-           psic(dffts%nlm(ig)) = CONJG( psic(dffts%nl(ig)) )
-        ENDDO
-        !$omp end do nowait
-     ENDIF
-  !$omp end parallel
-  !
-  CALL invfft ('Rho', psic, dffts)
+  CALL rho_g2r( dffts, drho%of_g(:,1), auxr )
   !
   !$omp parallel do
   DO ir = 1, dffts%nnr
-     psic(ir) = psic(ir) * alpha(ir)
+     auxr(ir) = auxr(ir) * alpha(ir)
   ENDDO
   !$omp end parallel do
   !
-  CALL fwfft ('Rho', psic, dffts)
+  CALL rho_r2g( dffts, auxr, auxg )
   !
   IF ( lgcscf ) THEN
      !
      !$omp parallel do
      DO ig = 1, ngm0
-        dv(ig) = psic(dffts%nl(ig)) * ( gg(ig) + bgg0 ) * tpiba2
-        v(ig,1)= psic(dffts%nl(ig)) * ( gg(ig) + bgg0 ) / ( gg(ig) + agg0 + bgg0 )
+        dv(ig) = auxg(ig,1) * ( gg(ig) + bgg0 ) * tpiba2
+        v(ig,1)= auxg(ig,1) * ( gg(ig) + bgg0 ) / ( gg(ig) + agg0 + bgg0 )
      ENDDO
      !$omp end parallel do
      !
@@ -613,8 +592,8 @@ SUBROUTINE approx_screening2( drho, rhobest )
      !
      !$omp parallel do
      DO ig = 1, ngm0
-        dv(ig) = psic(dffts%nl(ig)) * gg(ig) * tpiba2
-        v(ig,1)= psic(dffts%nl(ig)) * gg(ig) / ( gg(ig) + agg0 )
+        dv(ig) = auxg(ig,1) * gg(ig) * tpiba2
+        v(ig,1)= auxg(ig,1) * gg(ig) / ( gg(ig) + agg0 )
      ENDDO
      !$omp end parallel do
      !
@@ -629,40 +608,30 @@ SUBROUTINE approx_screening2( drho, rhobest )
      ! ... generate the vector w
      !     
      !$omp parallel
-        CALL threaded_barrier_memset(psic, 0.0_DP, dffts%nnr*2)
         !$omp do
         DO ig = 1, ngm0
            !
            w(ig,m) = fpi * e2 * v(ig,m)
            !
-           psic(dffts%nl(ig)) = v(ig,m)
         ENDDO
         !$omp end do nowait
-        !
-        IF ( gamma_only ) THEN
-           !$omp do
-           DO ig = 1, ngm0
-              psic(dffts%nlm(ig)) = CONJG( psic(dffts%nl(ig)) )
-           ENDDO
-           !$omp end do nowait
-        ENDIF
      !$omp end parallel
      !
-     CALL invfft ('Rho', psic, dffts)
+     CALL rho_g2r( dffts, v(:,m), auxr )
      !
      !$omp parallel do
      DO ir = 1, dffts%nnr
-        psic(ir) = psic(ir) * alpha(ir)
+        auxr(ir) = auxr(ir) * alpha(ir)
      ENDDO
      !$omp end parallel do
      !
-     CALL fwfft ('Rho', psic, dffts)
+     CALL rho_r2g( dffts, auxr, auxg )
      !
      IF ( lgcscf ) THEN
         !
         !$omp parallel do
         DO ig = 1, ngm0
-           w(ig,m) = w(ig,m) + ( gg(ig) + bgg0 ) * tpiba2 * psic(dffts%nl(ig))
+           w(ig,m) = w(ig,m) + ( gg(ig) + bgg0 ) * tpiba2 * auxg(ig,1)
         ENDDO
         !$omp end parallel do
         !
@@ -670,7 +639,7 @@ SUBROUTINE approx_screening2( drho, rhobest )
         !
         !$omp parallel do
         DO ig = 1, ngm0
-           w(ig,m) = w(ig,m) + gg(ig) * tpiba2 * psic(dffts%nl(ig))
+           w(ig,m) = w(ig,m) + gg(ig) * tpiba2 * auxg(ig,1)
         ENDDO
         !$omp end parallel do
         !
@@ -759,6 +728,7 @@ SUBROUTINE approx_screening2( drho, rhobest )
            !
         !$omp end parallel
         !
+        DEALLOCATE( auxr, auxg )
         DEALLOCATE( alpha, v, w, dv, vbest, wbest )
         !
         EXIT repeat_loop
