@@ -25,9 +25,10 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   USE scf,              ONLY : scf_type
   USE cell_base,        ONLY : alat
   USE io_global,        ONLY : stdout
-  USE control_flags,    ONLY : ts_vdw, mbd_vdw
+  USE control_flags,    ONLY : ts_vdw, mbd_vdw, sic
   USE tsvdw_module,     ONLY : tsvdw_calculate, UtsvdW
   USE libmbd_interface, ONLY : mbd_interface
+  USE sic_mod,          ONLY : add_vsic
   !
   IMPLICIT NONE
   !
@@ -138,6 +139,8 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
     call mbd_interface() ! self-consistent but only up to TS level
   END IF
   !
+  IF (sic) CALL add_vsic(rho, rho_core, rhog_core, v)
+  !
   CALL stop_clock( 'v_of_rho' )
   !
   RETURN
@@ -184,7 +187,7 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
   !
   REAL(DP) :: zeta, rh, sgn_is
   REAL(DP) :: etxc0, vtxc0
-  INTEGER  :: k, ipol, is, np
+  INTEGER  :: k, ipol, is, np, dfftp_nnr
   LOGICAL  :: lda_gga_terms
   !
   REAL(DP), ALLOCATABLE :: ex(:), ec(:), v0(:,:)
@@ -208,6 +211,7 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
   fac = 1.D0 / DBLE( nspin )
   np = 1
   IF (nspin==2) np=3
+  dfftp_nnr = dfftp%nnr !to avoid unnecessary copies in acc loop
   !
   !$acc data copyin( rho ) copyout( kedtaur, v )
   !
@@ -239,7 +243,7 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
   !
   !$acc parallel loop collapse(2) present(rho)
   DO is = 1, nspin
-    DO k = 1, dfftp%nnr
+    DO k = 1, dfftp_nnr
       tau(k,is) = rho%kin_r(k,is)/e2
     ENDDO
   ENDDO
@@ -251,12 +255,11 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
   !$acc data create( ex, ec, v1x, v2x, v3x, v1c, v2c, v3c )
   IF (nspin == 1) THEN
     !
-    CALL xc_metagcx( dfftp%nnr, 1, np, rho%of_r, grho, tau, ex, ec, &
+    CALL xc_metagcx( dfftp_nnr, 1, np, rho%of_r, grho, tau, ex, ec, &
                      v1x, v2x, v3x, v1c, v2c, v3c, gpu_args_=.TRUE. )
     !
-    !$acc parallel loop reduction(+:etxc,vtxc) reduction(-:rhoneg1,rhoneg2) &
-    !$acc&              present(rho)
-    DO k = 1, dfftp%nnr
+    !$acc parallel loop reduction(+:etxc,vtxc,rhoneg1,rhoneg2) present(rho)
+    DO k = 1, dfftp_nnr
        !
        v(k,1) = (v1x(k,1)+v1c(k,1)) * e2
        !
@@ -280,18 +283,18 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
     !$acc data create( rho_updw )
     !
     !$acc parallel loop present(rho)
-    DO k = 1, dfftp%nnr  
+    DO k = 1, dfftp_nnr  
         rho_updw(k,1) = ( rho%of_r(k,1) + rho%of_r(k,2) ) * 0.5d0
         rho_updw(k,2) = ( rho%of_r(k,1) - rho%of_r(k,2) ) * 0.5d0
     ENDDO
     !
-    CALL xc_metagcx( dfftp%nnr, 2, np, rho_updw, grho, tau, ex, ec, &
+    CALL xc_metagcx( dfftp_nnr, 2, np, rho_updw, grho, tau, ex, ec, &
                      v1x, v2x, v3x, v1c, v2c, v3c, gpu_args_=.TRUE. )
     !
     ! ... first term of the gradient correction : D(rho*Exc)/D(rho)
     !
-    !$acc parallel loop reduction(+:etxc,vtxc) reduction(-:rhoneg1,rhoneg2)
-    DO k = 1, dfftp%nnr
+    !$acc parallel loop reduction(+:etxc,vtxc,rhoneg1,rhoneg2)
+    DO k = 1, dfftp_nnr
        !
        v(k,1) = (v1x(k,1) + v1c(k,1)) * e2
        v(k,2) = (v1x(k,2) + v1c(k,2)) * e2
@@ -336,8 +339,8 @@ SUBROUTINE v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v, kedtaur )
      !
      sgn_is = (-1.d0)**(is+1)
      !
-     !$acc parallel loop reduction(-:vtxc) present(rho)
-     DO k = 1, dfftp%nnr
+     !$acc parallel loop reduction(+:vtxc) present(rho)
+     DO k = 1, dfftp_nnr
        v(k,is) = v(k,is) - dh(k)
        vtxc = vtxc - dh(k) * ( rho%of_r(k,1) + sgn_is*rho%of_r(k,nspin) )*0.5D0
      ENDDO
@@ -444,7 +447,7 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
     ! local correlation energy
     ! local exchange potential
     ! local correlation potential
-  INTEGER :: ir, ipol
+  INTEGER :: ir, ipol, dfftp_nnr
     ! counter on mesh points
     ! counter on polarization components
     ! number of mesh points (=dfftp%nnr)
@@ -453,6 +456,8 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   !
   CALL start_clock( 'v_xc' )
   !
+  dfftp_nnr = dfftp%nnr !to avoid unnecessary copies in acc loop
+  !
   etxc = 0.D0 ;  rhoneg1 = 0.D0
   vtxc = 0.D0 ;  rhoneg2 = 0.D0
   !
@@ -464,17 +469,17 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   !$acc data create( ex, ec, vx, vc )
   !
   !$acc parallel loop
-  DO ir = 1, dfftp%nnr
+  DO ir = 1, dfftp_nnr
     rho%of_r(ir,1) = rho%of_r(ir,1) + rho_core(ir)
   ENDDO
   !
   IF ( nspin == 1 .OR. ( nspin == 4 .AND. .NOT. domag ) ) THEN
      ! ... spin-unpolarized case
      !
-     CALL xc( dfftp%nnr, 1, 1, rho%of_r, ex, ec, vx, vc, gpu_args_=.TRUE. )
+     CALL xc( dfftp_nnr, 1, 1, rho%of_r, ex, ec, vx, vc, gpu_args_=.TRUE. )
      !
-     !$acc parallel loop reduction(+:etxc,vtxc) reduction(-:rhoneg1) present(rho)
-     DO ir = 1, dfftp%nnr
+     !$acc parallel loop reduction(+:etxc,vtxc,rhoneg1) present(rho)
+     DO ir = 1, dfftp_nnr
         v(ir,1) = e2*( vx(ir,1) + vc(ir,1) )
         etxc = etxc + e2*( ex(ir) + ec(ir) )*rho%of_r(ir,1)
         rho%of_r(ir,1) = rho%of_r(ir,1) - rho_core(ir)
@@ -486,11 +491,11 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   ELSEIF ( nspin == 2 ) THEN
      ! ... spin-polarized case
      !
-     CALL xc( dfftp%nnr, 2, 2, rho%of_r, ex, ec, vx, vc, gpu_args_=.TRUE. )
+     CALL xc( dfftp_nnr, 2, 2, rho%of_r, ex, ec, vx, vc, gpu_args_=.TRUE. )
      !
-     !$acc parallel loop reduction(+:etxc,vtxc) reduction(-:rhoneg1,rhoneg2) &
+     !$acc parallel loop reduction(+:etxc,vtxc,rhoneg1,rhoneg2) &
      !$acc&              present(rho)
-     DO ir = 1, dfftp%nnr
+     DO ir = 1, dfftp_nnr
         v(ir,1) = e2*( vx(ir,1) + vc(ir,1) )
         v(ir,2) = e2*( vx(ir,2) + vc(ir,2) )
         etxc = etxc + e2*( (ex(ir) + ec(ir))*rho%of_r(ir,1) )
@@ -507,11 +512,10 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
    ELSEIF ( nspin == 4 ) THEN
       ! ... noncollinear case
       !
-      CALL xc( dfftp%nnr, 4, 2, rho%of_r, ex, ec, vx, vc, gpu_args_=.TRUE. )
+      CALL xc( dfftp_nnr, 4, 2, rho%of_r, ex, ec, vx, vc, gpu_args_=.TRUE. )
       !
-      !$acc parallel loop reduction(+:etxc,vtxc) reduction(-:rhoneg1,rhoneg2) &
-      !$acc&              present(rho)
-      DO ir = 1, dfftp%nnr
+      !$acc parallel loop reduction(+:etxc,vtxc,rhoneg1,rhoneg2) present(rho)
+      DO ir = 1, dfftp_nnr
          arho = ABS( rho%of_r(ir,1) )
          IF ( arho < vanishing_charge ) THEN
            v(ir,1) = 0.d0 ;  v(ir,2) = 0.d0
@@ -567,6 +571,9 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   !$acc end data
   !$acc end data
   !
+  ! ... to avoid NaN in some rare cases (see summations in subroutine delta_e)
+  IF ( nspin==4 .AND. .NOT.domag ) v(:,2:nspin) = 0.D0
+  !
   ! ... add non local corrections (if any)
   !
   IF ( dft_is_nonlocc() ) CALL nlc( rho%of_r, rho_core, nspin, etxc, vtxc, v )
@@ -588,7 +595,7 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
   USE constants,         ONLY : fpi, e2
   USE kinds,             ONLY : DP
   USE fft_base,          ONLY : dfftp
-  USE fft_interfaces,    ONLY : invfft
+  USE fft_rho,           ONLY : rho_g2r
   USE gvect,             ONLY : ngm, gg, gstart
   USE lsda_mod,          ONLY : nspin
   USE cell_base,         ONLY : omega, tpiba2
@@ -612,7 +619,7 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
   !  ... local variables
   !
   REAL(DP)              :: fac
-  REAL(DP), ALLOCATABLE :: aux1(:,:)
+  REAL(DP), ALLOCATABLE :: aux1(:,:), vh(:)
   REAL(DP)              :: rgtot_re, rgtot_im, eh_corr
   INTEGER               :: is, ig
   COMPLEX(DP), ALLOCATABLE :: aux(:), rgtot(:), vaux(:)
@@ -620,24 +627,24 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
   !
   CALL start_clock( 'v_h' )
   !
-  ALLOCATE( aux( dfftp%nnr ), aux1( 2, ngm ) )
+  ALLOCATE( aux(dfftp%nnr), aux1(2,ngm), vh(dfftp%nnr) )
   charge = 0.D0
   !
   IF ( gstart == 2 ) THEN
      !
      charge = omega*REAL( rhog(1) )
      !
-  END IF
+  ENDIF
   !
-  CALL mp_sum(  charge , intra_bgrp_comm )
+  CALL mp_sum( charge, intra_bgrp_comm )
   !
   ! ... calculate hartree potential in G-space (NB: V(G=0)=0 )
   !
-  IF ( do_comp_esm .and. ( esm_bc .ne. 'pbc' ) ) THEN
+  IF ( do_comp_esm .AND. ( esm_bc .NE. 'pbc' ) ) THEN
      !
      ! ... calculate modified Hartree potential for ESM
      !
-     CALL esm_hartree (rhog, ehart, aux)
+     CALL esm_hartree( rhog, ehart, aux )
      !
   ELSE
      !
@@ -674,56 +681,49 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
         !
         ehart = ehart * omega
         !
-     ELSE 
+     ELSE
         !
         ehart = ehart * 0.5D0 * omega
         !
-     END IF
-     ! 
-     if (do_comp_mt) then
-        ALLOCATE( vaux( ngm ), rgtot(ngm) )
+     ENDIF
+     !
+     IF (do_comp_mt) THEN
+        ALLOCATE( vaux(ngm), rgtot(ngm) )
         rgtot(:) = rhog(:)
-        CALL wg_corr_h (omega, ngm, rgtot, vaux, eh_corr)
+        CALL wg_corr_h( omega, ngm, rgtot, vaux, eh_corr )
         aux1(1,1:ngm) = aux1(1,1:ngm) + REAL( vaux(1:ngm))
         aux1(2,1:ngm) = aux1(2,1:ngm) + AIMAG(vaux(1:ngm))
         ehart = ehart + eh_corr
         DEALLOCATE( rgtot, vaux )
-     end if
+     ENDIF
      !
-     CALL mp_sum(  ehart , intra_bgrp_comm )
-     ! 
-     aux(:) = 0.D0
+     CALL mp_sum( ehart, intra_bgrp_comm )
      !
-     aux(dfftp%nl(1:ngm)) = CMPLX( aux1(1,1:ngm), aux1(2,1:ngm), KIND=dp )
+     aux(1:ngm) = CMPLX( aux1(1,1:ngm), aux1(2,1:ngm), KIND=DP )
      !
-     IF ( gamma_only ) THEN
-        !
-        aux(dfftp%nlm(1:ngm)) = CMPLX( aux1(1,1:ngm), -aux1(2,1:ngm), KIND=dp )
-        !
-     END IF
-  END IF
+  ENDIF
   !
-  ! ... transform hartree potential to real space
+  ! ... transform Hartree potential to real space
   !
-  CALL invfft('Rho', aux, dfftp)
+  CALL rho_g2r( dfftp, aux, vh )
   !
-  ! ... add hartree potential to the xc potential
+  ! ... add Hartree potential to the xc potential
   !
   IF ( nspin == 4 ) THEN
      !
-     v(:,1) = v(:,1) + DBLE(aux(:))
+     v(:,1) = v(:,1) + vh(:)
      !
   ELSE
      !
      DO is = 1, nspin
         !
-        v(:,is) = v(:,is) + DBLE(aux(:))
+        v(:,is) = v(:,is) + vh(:)
         !
-     END DO
+     ENDDO
      !
-  END IF
+  ENDIF
   !
-  DEALLOCATE( aux, aux1 )
+  DEALLOCATE( aux, aux1, vh )
   !
   CALL stop_clock( 'v_h' )
   !
@@ -890,8 +890,11 @@ SUBROUTINE v_hubbard_b (ns, v_hub, eth)
      !
      nt = ityp (na)
      !
-     IF (Hubbard_J0(nt).NE.0.d0 .OR. Hubbard_beta(nt).NE.0.d0) &
-     CALL errore('v_hubbard_b', 'Hubbard_J0 and Hubbard_beta are not supported',1) 
+     IF (Hubbard_J0(nt).NE.0.d0) &
+          CALL errore('v_hubbard_b', 'J0 is not supported in DFT+U with multiple channels per atomic type',1)
+     !
+     IF (Hubbard_beta(nt).NE.0.d0) &
+     CALL errore('v_hubbard_b', 'Hubbard_beta is not supported in DFT+U with multiple channels per atomic type',1) 
      !
      IF (is_hubbard_back(nt)) THEN
         !
