@@ -15,7 +15,7 @@ SUBROUTINE vhpsi( ldap, np, mps, psip, hpsi )
   USE kinds,         ONLY : DP
   USE becmod,        ONLY : bec_type, calbec, allocate_bec_type, &
                             deallocate_bec_type
-  USE ldaU,          ONLY : Hubbard_l, is_Hubbard,   &
+  USE ldaU,          ONLY : Hubbard_l, is_hubbard,   &
                             nwfcU, wfcU, offsetU, lda_plus_u_kind, &
                             is_hubbard_back, Hubbard_l2, offsetU_back, &
                             backall, offsetU_back1
@@ -567,7 +567,7 @@ SUBROUTINE vhpsi_nc( ldap, np, mps, psip, hpsi )
   !! Noncollinear version of \(\texttt{vhpsi} routine (A. Smogunov).
   !
   USE kinds,            ONLY: DP
-  USE ldaU,             ONLY: Hubbard_lmax, Hubbard_l, is_Hubbard, nwfcU, &
+  USE ldaU,             ONLY: Hubbard_lmax, Hubbard_l, is_hubbard, nwfcU, &
                               wfcU, offsetU
   USE scf,              ONLY: v
   USE ions_base,        ONLY: nat, ntyp => nsp, ityp
