@@ -46,7 +46,7 @@ program all_currents
 
    use dynamics_module, only: vel
    use ions_base, ONLY: tau, tau_format, nat
-   USE control_flags, ONLY: ethr
+   USE control_flags, ONLY: ethr, use_gpu
    USE extrapolation, ONLY: update_pot
 
 !from ../PW/src/pwscf.f90
@@ -122,13 +122,14 @@ program all_currents
    character(len=256) :: vel_input_units = 'PW', worker_id_char, format_string
    logical :: ec_test, hpsi_test ! activates tests for debugging purposes
    logical :: continue_not_converged ! don't stop the calculation if a step does not converge
+   LOGICAL,EXTERNAL :: check_gpu_support
    !from ../PW/src/pwscf.f90
    include 'laxlib.fh'
 
 !from ../PW/src/pwscf.f90
+   use_gpu = check_gpu_support()
+   if(use_gpu) Call errore('QEHeat', 'QEHeat with GPU NYI.', 1)
    CALL mp_startup( images_only=.true. )
-   CALL set_mpi_comm_4_solvers(intra_pool_comm, intra_bgrp_comm, &
-                               inter_bgrp_comm)
    CALL environment_start('QEHeat')
    call start_clock('all_currents')
    IF (ionode) THEN
